/*****************************************************************************

   RX_BLOQUEADOR_V2

   RECEPTOR P/ BLOQUEADOR DE VECULOS (PRESENA) VERSO 2

   USA PROTOCOLO  DO TIPO HOPPING CODE (MUDA OS VALORES DE CADA TRANSMISSO)

   Autor: Claudio Lrios

   Data: 02/08/2020

   Funciona em conjunto com transmissor "TX_BLOQUEADOR_V2"

   Permite gravar at 18 TX_BLOQUEADOR_V2 (com serial number diferentes)

   ATENO: No funciona com nenhum tipo de TX comerciais (os 'chaveirinhos')
   mesmo os que possuam circuito integrados como o HCS300,301,200,201,etc.
   Necessita montar os 'TX_BLOQUEADOR_V2' e grava-los com firmware fornecido.

   Sobre o funcionamento:
   A cada tansmisso ser enviado um cdigo diferente de 64 bits. Todos os bits
   so encriptados por uma KEY tambm de 64 bits.

   Para gravar um tx:

   1) Aperte o boto 'gravar_ser' do RX (receptor). Neste momento dever acender
      o led de programao por 20 segundos.

   2) No tx (transmissor): Aperte  o boto 'bt_env_ser'(ou coloque jumper) .
   O tx transmitir o serial number para o receptor. O led de programao ir
   apagar indicando que foi gravado o serial number no receptor. Pare de pressi-
   onar o boto 'bt_env_ser'( ou se for jumper, retire ele).

   3) Retire e recoloque a bateria para iniciar uma transmisso. Voc deve
   conferir que neste momento, o receptor acione o rele de sada.

   Importante: No receptor, o boto 'gravar_ser' tem mais uma funo.
   Se ele for apertado continuamente por mais de 10 segundos ir apagar todos os
   controles gravados na EEprom do PIC.

   Nota: O boto 'bt_env_ser' do tx serve somente para enviar o 'serial number'
   para o receptor em modo de gravao.
   Portanto, dever ficar alojado dentro da caixa do tx (podendo ser acessado
   talvez atravs de um pequeno furo, usando a ponta de um clips de papel para
   aperta-lo).
   Opcionalmente, pode ser usado um jumper ao invs de boto nesta funo.
   Altere a Key tanto no receptor como no transmissor e mantenha seu valor si-
   giloso (secreto).
   Importante: A Key deve ser igual nos txs e no rx. O serial number do tx deve
   ser diferente para cada tx (escolha a gosto os valores).


   Este arquivo  parte integrante do blog Picsource

    Uso didtico

******************************************************************************/


#include <12F675.h>
#device adc=10
#use delay(clock=4000000)
#fuses NOWDT,INTRC_IO, NOPROTECT, NOBROWNOUT, NOMCLR, NOCPD, NOPUT


//BYTE

#byte GPIO       = 0x05
#byte TRISIO     = 0x85
#byte ANSEL      = 0x9F
#byte CMCON      = 0x19
#byte OPTION_REG = 0x81
#byte WPU        = 0x95
#byte INTCON     = 0x0B
#byte TMR0       = 0x01
#byte TMR1H      = 0x0F
#byte TMR1L      = 0x0E
#byte T1CON      = 0x10

//BIT

#bit   rf_in    = 0x5.0
#bit   led      = 0x5.5
#bit   btn      = 0x5.3 //boto gravar/apagar tudo
#bit   rele     = 0x5.2 //entrada dos interruptores
#bit   led_ad   = 0x5.1 //led de advertencia
#bit   led_bloq = 0x5.4 //led bloqueado

#bit   T0IF     =  0X0B.2
#bit   TMR1IF   =  0X0C.0
//chave secreta de Encriptao (Secret Encriptacion Key)
// Use os mesmos valores no tx e no rx

#define KEY0  0X12
#define KEY1  0X34
#define KEY2  0X56
#define KEY3  0X78
#define KEY4  0X9A
#define KEY5  0XBC
#define KEY6  0XDE
#define KEY7  0XF0

#define N_ROTATE 64  //rotao do encript


unsigned int1 flag0=0;
unsigned int1 flag_ok=0;
unsigned int1 flag_r=0;
unsigned int1 flag_inc=0;//flag incremento do contador encriptao EEprom
unsigned int8 buffer_Ser[8];
unsigned int8 buffer_Key[8];
unsigned int8 hc;
unsigned int8 lc;
unsigned int8 qb;
unsigned int8 auxa;
unsigned int8 auxb;
unsigned int8 auxc;
unsigned int8 auxd;
unsigned int8 cont_enk0;//salva contador encriptao lower
unsigned int8 cont_enk1;//salva contador encriptao higher
unsigned int8 cont_enk2;//salva contador encriptao upper
unsigned int8 cntap=0;
unsigned int16 cntinc=0;
unsigned int32 cntprog=0;
unsigned int32 accA;
unsigned int32 accB;
unsigned int16 temp_desl=0;

/******************************************************************************
   Array com os valores de substituio usados na decriptao de dados
*******************************************************************************/

//POSIO                 0  1  2  3  4  5   6   7  8   9  10 11  12 13 14 15
int8 const TABELA_Z[] = { 4, 7, 5, 9,10, 1, 11, 12, 0, 13, 14, 2, 15, 8, 6, 3};
/******************************************************************************
         Rotina de escrita na EEprom interna do PIC
*******************************************************************************/
void escreve_eep(char end, char valor){
  write_eeprom(end,valor);
}
/******************************************************************************
     Rotina de Delays
*******************************************************************************/
void delayp(){ delay_ms(300);}
void delay50ms(){delay_ms(50);}
/******************************************************************************
     Rotina de teste do boto de gravao/sincronizao ou apagamento
*******************************************************************************/

void testa_botao(){
#define TEMPO_APAGAMENTO 100

  if(btn==1)return;
  delay50ms();
  cntap=0;
  while(!btn){
    led=1;
    cntap++;
    delay50ms();
    if(cntap>TEMPO_APAGAMENTO){
      for(auxa=0;auxa<128;auxa++){
        escreve_eep(auxa,0xff); //seta todas as outras posies da EEprom
      }
     cntap=0;
     for(auxa=0;auxa<6;auxa++){//pisca led 5 vezes para indicar apagamento
        led=1;
        delayp();
        led=0;
        delayp();
      }
     while(!btn)delay_ms(100);//aguarda soltar boto
    }
  }
}

/******************************************************************************
         Rotina de Decriptao dos Dados Recebidos
*******************************************************************************
Rotina de decodificao utiliza uma chave secreta (key) de 64 bits.
Primeiro os dados recebidos sofrem uma substituio de valores a partir de uma
tabela. Em seguida rotaciona todos os 8 bytes para esquerda em 1 bit. Ento 
feita a aplicao de XOR com a chave. O processo se repete pelo nmero de rota-
es estabelecido. No final, temos o valor do serial number (4 bytes) + o
contador encripta (3 bytes) + situao dos botes (1 byte) . */

void decriptar(){
 if(flag_ok==0)return;
   //Carrega valores da Key de Encriptao
  buffer_Key[0]= KEY0;
  buffer_Key[1]= KEY1;
  buffer_Key[2]= KEY2;
  buffer_Key[3]= KEY3;
  buffer_Key[4]= KEY4;
  buffer_Key[5]= KEY5;
  buffer_Key[6]= KEY6;
  buffer_Key[7]= KEY7;

  for(auxa=0;auxa<N_ROTATE;auxa++){
    //rotina de substituio de valores
    for(auxb=0;auxb<8;auxb++){
      auxc=buffer_Ser[auxb] & 0x0f;
      buffer_Ser[auxb]= buffer_Ser[auxb] & 0xf0;//zera 4 bits low
      buffer_Ser[auxb]= buffer_Ser[auxb] | TABELA_Z[auxc] ;
    }

    rotate_left(buffer_Ser,8);//rotaciona 1 bits a esquerda
     //loop de encriptao

    for(auxb=0;auxb<8;auxb++){
      buffer_Ser[auxb]= buffer_Ser[auxb] ^ buffer_Key[auxb];
    }
  }
}

/******************************************************************************
       Rotina de Desligamento do Modo de Programao de Tx's por tempo
*******************************************************************************/
void temporiza_prog_mode(){
#define TEMPO_MAX_PROG_MODE 220000 //cerca de uns 20 segundos c/ led ligado
 if(led==0) return;
  cntprog++;
  if(cntprog==TEMPO_MAX_PROG_MODE){
   led=0;
   cntprog=0;
  }
}

/******************************************************************************
      Rotina de Incrementos de Registradores da Recepo de Dados
*******************************************************************************/
void incrementa_hc_lc(){
  if(rf_in==1){
    flag0=1;//flag de estado da entrada rf_in
    if(hc<255)hc++;//no deixa passar de 255
  }
  else{
    if(lc<255)lc++;//no deixa passar de 255
  }
 // delay_us(10);//delay 50
}
/******************************************************************************
         Rotina de Recepo de Dados do Receptor RF
*******************************************************************************/
void receber_dados(){
#define NUMERO_BITS_A_RECEBER 64
#define TMAX 20 //20
   if(flag_ok==1)return;

   if(flag0==0){
     incrementa_hc_lc();
   }
   else{//flag0==1
     if(rf_in==0){
       if(lc>TMAX){//define como intervalo entre transmisses


       qb=0;lc=0;hc=0;flag0=0;//reinicia contadores
       }
       else{

         rotate_right(buffer_Ser,8);
         if(lc>hc){
           buffer_Ser[7]= buffer_Ser[7] | 0x80;
         }
         else{
           buffer_Ser[7]= buffer_Ser[7] & 0x7f;
         }

         flag0=0;//prepara para pegar novo bit
         hc=0;
         lc=0;
         qb++;
         if(qb>=NUMERO_BITS_A_RECEBER){
          qb=0;
          if(flag_r==0){
            flag_r=1;
            for(auxb=0;auxb<8;auxb++){
              buffer_Key[auxb]=buffer_Ser[auxb];//salva recepo para comparao
            }

          }
          else{
            for(auxb=0;auxb<8;auxb++){
              if(buffer_Key[auxb]!=buffer_Ser[auxb])break;
            }
            if(auxb==8){
            flag_ok=1;//recebeu uma transmisso completa
            }
            flag_r=0;
          }
         }
       }//else
     } //if(rf_in==0){
     else{
       incrementa_hc_lc();
     }
   }// else{//flag0==1
}//void testa_rx(){


/******************************************************************************
      Verifica se tx foi gravado e retorna seu endereo na EEprom do PIC
*******************************************************************************/
char procura_tx_eep(){
  //procura de 7 em 7 bytes mas compara apenas os 4 primeiros bytes de 7
  for (auxa = 0; auxa < 120; auxa++){
     for (auxb=0;auxb<4;auxb++){
        if(buffer_Ser[auxb] != read_eeprom(auxa + auxb))break;
     }
     if(auxb==4){return auxa;}  //encontrou tx: retorna o endereo inicial do tx
     auxa = auxa + 6;
  }
  return 0xAA;//no encontrado
}

/******************************************************************************
         Rotina de Procura por Posies Vazias na EEprom (para gravar novo tx)
*******************************************************************************/
char procura_gpos(){
  for(auxa=0; auxa<120;auxa++){
    for(auxb=0; auxb<8;auxb++){
      if(read_eeprom(auxa+auxb)!=0xff)break;
    }
    if(auxb==8) return auxa;
    auxa=auxa + 6;
  }
  return 0xff;
}

/******************************************************************************
         Rotina de Gravao/ressincronizao de Transmissores
*******************************************************************************/
/* Quando em 'modo gravao' (led aceso) ir aguardar a recepo do serial
   number do tx a gravar. O usurio deve apertar somente o boto 'bt_env_ser'
   do tx para que este receptor faa o aprendizado e grave na eeprom.         */
void gravar_syn_tx(){
  if(flag_ok==0)return;
  auxa=0;
  for(auxb=0;auxb<7;auxb++){
  auxa= auxa + buffer_Ser[auxb];//soma bytes recebidos
  }
  if(auxa!=buffer_Ser[7]){//verifica o 'test sum'
    flag_ok=0;
    return;
  }
  auxd = procura_tx_eep();//procura tx na eeprom, se j esta gravado
  if(auxd<128){//j est gravado,ir somente ressincronizar o contador

  for(auxa=4;auxa<7;auxa++){//grava 3 bytes do contador na eeprom
        escreve_eep(auxd+auxa,buffer_Ser[auxa]);
      }
      led=0; delayp(); led=1; delayp();//piscada adicional para ressincronizao
  }
  else{
    auxd=procura_gpos();//procura posio livre para gravar
    if(auxd==0xff){//no tem mais espao, sinaliza erro com 5 piscadas
      for(auxb=0;auxb<5;auxb++){
      led=0; delayp(); led=1; delayp();
      }
    }
    else{
      for(auxa=0;auxa<7;auxa++){//grava 7 bytes na eeprom
        escreve_eep(auxd+auxa,buffer_Ser[auxa]);
      }
    }
  }

  led=0; delayp(); led=1; delayp(); led=0;// 1 piscada para gravao correta
  flag_ok=0; //libera nova recepo
}

/******************************************************************************
         Rotina de Acionamento do rele
*******************************************************************************/

void acionar_rele(){
#define MAX_DESSINCRONIZACAO 1//ajusta taxa de dessincronizao mxima

/* Se o controle remoto transmissor tiver os seus botes apertados por mais de
   5 vezes e estando longe do receptor (portanto, sem acionar o receptor) ento
   ficar dessincronizado. Na primeira vez que acionar o boto perto do receptor
   no  ligar o rele, mas ir ressincronizar ao trmino da recepo. A proxima
   transmisso, se correta, ir acionar o rele. Isto previne ataque a 'fora
   bruta', pois necessitar 2 transmisses corretas sucessivas para ativar o
   rele. O objetivo do contador de encriptao tambm ser salvo no receptor 
   poder bloquear as transmisses corretas anteriores. Isto evita que algum
   escaneie a transmisso e tente usa-la para acionar o receptor.
   Contador encript do tx recebido no buffer_Ser[4],[5],[6].          */

   if(flag_ok==0)return;
   auxd = procura_tx_eep();


   if(auxd < 128){//foi encontrado um tx cadastrado (auxd tem o endereo dele)

 //Salva valor do contador encript recebido do tx p/ depois salvar na EEprom
     cont_enk0=buffer_Ser[4];//contador encript lower
     cont_enk1=buffer_Ser[5];//contador encript higher
     cont_enk2=buffer_Ser[6];//contador encript upper

  //   flag_inc=1; //sinaliza p/ processo de incremento do contadorEncript
     cntinc=0;//reinicia tempo de espera ps termino de recepo
 //accA= Varivel de 32 bits p/ receber o valor do contador encript deste rx
     accA = read_eeprom(auxd+4);//obtem cnt lower do receptor
     accA = accA | (int32)(read_eeprom(auxd+5))<<8;//obtem cnt higher do receptor
     accA = accA | (int32)(read_eeprom(auxd+6))<<16;//obtem cnt upper

//accB= Varivel de 32 bits p/ receber o valor do contador encript do tx
     accB = cont_enk0;
     accB = accB | (int32)(cont_enk1)<<8;
     accB = accB | (int32)(cont_enk2)<<16;


  /*Comparao p/ impedir que transmisses corretas anteriores acionem o rele
    Evita que uma transmisso anterior escaneada consiga acionar o rele.
    valor do contador recebido tem que ser maior que o salvo no receptor.   */
    if(accB>accA){
      accA = accB - accA;//verifica taxa de dessincronizao
      if(accA <= MAX_DESSINCRONIZACAO){ //testa grau de dessincronizao
       rele=1;//
       led_ad=0;//desliga led de advertncia      
       led_bloq=0;//desliga led de bloqueio       
       temp_desl=0;//zera contador de tempo
     }
     flag_inc=1;//sinaliza p/ processo de incremento do contadorEncript
   }//if(accB > accA){
  }
 flag_ok=0;//libera nova recepo
}
/******************************************************************************
  Rotina de Deteco do Fim das Transmisses p/ Salvar Valor do Contador Encript
*******************************************************************************/
/* Somente ao termino de uma transmisso (curta ou longa dependendo do tempo que
o boto do tx fique apertado pelo usurio)  que ser atualizado o contador
encript. Cada vez que recebe uma transmisso correta o contador cntinc  zerado.
Quando para a transmisso ele consegue incrementar a ponto de permitir que seja
feita a atualizao do novo valor do contador encript.  */

void inc_cont_eep(){
#define VALOR_DELAY_INCREMENTO 2000
  if(flag_inc==0) return;
  cntinc++;
  if(cntinc>VALOR_DELAY_INCREMENTO){
    //ressincroniza automticamente

     if(read_eeprom(auxd+4)!=cont_enk0){//salva apenas se valor for diferente
       escreve_eep(auxd+4,cont_enk0);//salva cnt lower
     }
     if(read_eeprom(auxd+5)!=cont_enk1){//salva apenas se valor for diferente
      escreve_eep(auxd+5,cont_enk1);//salva cnt higher
     }
     if(read_eeprom(auxd+6)!=cont_enk2){//salva apenas se valor for diferente
      escreve_eep(auxd+6,cont_enk2);//salva cnt uppe
     }
     flag_inc=0;//impede entrada nesta rotina at uma nova recepo ok
  }
}
/******************************************************************************
         Rotina de Desligamento do rele com indicao por leds
*******************************************************************************/
void temporiza_rele(){
//Frmula: VALOR_ADVERTENCIA = tempo em segundos/(65538 *8 * 0,000001)
// tempo para 60 segundos:
//  VALOR_ADVERTENCIA = 60/(65538 * 8 * 0,000001)
//  VALOR_ADVERTENCIA = 114,44  (use 114)
#define VALOR_ADVERTENCIA  171  //90 segundos
#define VALOR_BLOQUEIO     (VALOR_ADVERTENCIA * 2)//o dobro do tempo do anterior
   if(rele==0)return;
   if(TMR1IF==0)return;
   TMR1IF=0;
   temp_desl++;
  if(temp_desl>VALOR_ADVERTENCIA)led_ad=1; //seta flag pisca led de advertncia
  if(temp_desl>VALOR_BLOQUEIO){
    rele=0;//desliga rele
    led_ad=0;//desliga led de advertncia    
    led_bloq=1;//liga led de advertncia    
    temp_desl=0;
  }
}


/******************************************************************************
                      Rotina Main
*******************************************************************************/
void main() {

   OPTION_REG = 0;
   T1CON=0b00110001;//liga TIMER 1 prescaller/8
   TMR1IF=0;
   CMCON = 7;//desliga comparadores
   GPIO = 0;
   TRISIO = 0b001001;
   ANSEL = 0;
   lc=0;
   hc=0;
   flag0=0;
   qb=0;
   flag_ok=0;
   flag_r=0;

/******************************************************************************
                      Lao de Repetio
*******************************************************************************/
   for(;;){
    testa_botao();//testa boto gravao/apagamento e ressincronizao
    temporiza_prog_mode();//sai do modo de programao por tempo
    receber_dados();//realiza a recepo dos dados vindos do tx

    if(led==1){//testa se modo de gravao de tx est em andamento
      gravar_syn_tx();//rotina de gravao/ressincronizao de tx
    }
    else{//modo normal
      decriptar();//decodifica a transmisso recebida
      acionar_rele();//aciona rele se encontrou serial number do tx j gravado
    }
    inc_cont_eep();//incrementa contador de encriptao na EEprom
    //rotinas definidas pelo usurio no cabealho: Pulso ou Reteno
    temporiza_rele();//temporiza rele para desligamento caso no receba tx    
   }// for(;;)
}//main()

//------------------------------------------------------------------------------
//                           fim do programa

//------------------------------------------------------------------------------

