/*****************************************************************************

   TX_BLOQUEADOR_V2

   Autor: Claudio Lrios

   Data: 03/08/2020

   Transmissor para ser usado em conjunto com o receptor 'RX_PRESENCA_V2'

   Funcionamento: A cada 45 segundos este dispositivo emite uma transmisso de
   forma automtica. O usurio dever ficar com este transmissor junto do seu 
   corpo. No deve ficar junto com as chaves do veculo, mas como um 
   'chaveirinho' separado.
   Ele transmitir para o receptor instalado no veculo (moto ou carro).
   O receptor (RX_BLOQUEADOR_V2) ir  acionar um rele  ou manter ele ligado 
   cada vez que recebe a transmisso do tx.
   Este rele dever controlar a bomba de combustivel ou a bobina de ignio do
   veculo. O objetivo  imobilizar o veculo em caso de roubo, quando o dono do
   veculo  liberado e o veculo levado. Se ele continuar em poder dos meliantes
   o sistema permitir que tudo funcione normalmente.

   Quando o usurio se afastar do receptor, a sua transmisso no ser mais captada
   levando o receptor, aps um tempo definido, desligar o rele do receptor e 
   imobilizar o veculo.

   Ateno: O uso de dispositivos de segurana que cortem o combustvel ou a
   ignio do veculo podem causar acidentes. O uso deste dispositivo  por
   conta e risco do usurio. Verifique a legislao de transito de sua
   localidade sobre a permisso do uso de dispositivos como este.
   Leve em conta a corrente que passar pelo rele. Use um de corrente apropriada
   a carga que estar controlando. 
   A transmisso ser sempre diferente da anterior. Usa processo de encriptao
   com XOR + tabela de substituio.
   Este tx tem que ser cadastrado no receptor.
   Para isto:
   1)Aperte o boto 'gravar_ser' do receptor. O led acender. Nesta situao,
     o receptor fica aguardando um transmisso do nmero serial do transmissor.
   2)No transmissor, aperte o boto 'env_ser'. O led do receptor deve piscar e
     apagar, indicando o aprendizado. 
   3)Desligue e religue a alimentao do transmissor. Dever transmitir e 
     acionar o rele do receptor, liberando o funcionamento do veculo.

   Quando estiver longe do veculo ou no o estiver usando, desligue o 
   transmissor para aumentar o tempo de vida til das pilhas.
   Tenha sempre no veculo algumas pilhas reservas.
   Poder tambm instalar um interruptor para eliminar o sistema em caso 
   de pane. Dever ficar em um lugar escondido no veculo.


   Uso didtico


   Este arquivo  parte integrante do blog Picsource   

******************************************************************************/
#include <12F675.h>
#device adc=10
#use delay(clock=4000000)
#fuses WDT,INTRC_IO, NOPROTECT, NOBROWNOUT, NOMCLR, NOCPD, NOPUT
#rom 0x2100 = {0,0,0} //

//BYTE

#byte GPIO   = 0x05
#byte TRISIO = 0x85
#byte ANSEL  = 0x9f
#byte CMCON  = 0x19
#byte OPTION_REG = 0x81
#byte WPU  = 0X95
#byte IOC  = 0X96
#byte INTCON  = 0X0B

//BIT pinos do pic (tx mode)

#bit   rf_out  = 0x5.0 //pino 7
#bit   bt_env_ser = 0x5.5//pino 2 boto p/ envio do serial number

#bit   TO         = 0X03.4 //bit TO no STATUS

//Escolha os valores abaixo diferente para cada cpia de tx controle remoto
#define SER0  0X89
#define SER1  0XAB
#define SER2  0XCD
#define SER3  0XEF
//#define SER4  lower  cnt
//#define SER5  higher cnt
//#define SER6  upper  cnt
//#define SER7  situao dos botes bit 0 e 1

//chave secreta (Key) - Use ela com os mesmos valores no tx e no rx
#define KEY0  0X12
#define KEY1  0X34
#define KEY2  0X56
#define KEY3  0X78
#define KEY4  0X9A
#define KEY5  0XBC
#define KEY6  0XDE
#define KEY7  0XF0

//defines
#define N_ROTATE 64  //rotao do encript
#define Q_TRANSMITIDO 6  //nmero de vezes que ocorre uma mesma transmisso

//array de constantes   0 1  2  3  4  5  6  7  8  9 10 11 12 13 14 15
int8 const TABELAK[] = {8,5,11,15, 0, 2,14, 1,13, 3, 4, 6, 7, 9,10,12};

//variveis
unsigned int8 buffer_Ser[8];
unsigned int8 buffer_Key[8];
unsigned int8 cntwdt;


/*******************************************************************************
                    Sub-rotinas
*******************************************************************************/


/*******************************************************************************
                    Encriptao
*******************************************************************************/
//rotina de encriptao
void tx_encripta(){
int8 auxa,auxb,auxc;
  buffer_Ser[0]=SER0;
  buffer_Ser[1]=SER1;
  buffer_Ser[2]=SER2;
  buffer_Ser[3]=SER3;
  buffer_Ser[4]= READ_EEPROM(0); //cnt lower
  buffer_Ser[5]= READ_EEPROM(1); //cnt higher
  buffer_Ser[6]= READ_EEPROM(2); //cnt upper
  //buffer[7] ajustado com botes no bits 0 e 1

  //Carrega valores da Key de Encriptao
  buffer_Key[0]= KEY0;
  buffer_Key[1]= KEY1;
  buffer_Key[2]= KEY2;
  buffer_Key[3]= KEY3;
  buffer_Key[4]= KEY4;
  buffer_Key[5]= KEY5;
  buffer_Key[6]= KEY6;
  buffer_Key[7]= KEY7;




  //loop de encriptao
  for(auxa=0;auxa<N_ROTATE;auxa++){
    restart_wdt();
    for(auxb=0;auxb<8;auxb++){
      buffer_Ser[auxb]= buffer_Ser[auxb] ^ buffer_Key[auxb];
    }

    rotate_right(buffer_Ser,8);//rotaciona 1 bits a direita

    //rotina de substituio de valores
    for(auxb=0;auxb<8;auxb++){
      auxc=buffer_Ser[auxb] & 0x0f;
      buffer_Ser[auxb]= buffer_Ser[auxb] & 0xf0;//zera 4 bits low
      buffer_Ser[auxb]= buffer_Ser[auxb] | TABELAK[auxc] ;
    }

  }

}

void incrementa_cnt(){
int8 auxc;
//incrementa contador da encriptao
  restart_wdt();
     auxc=read_eeprom(0);
     auxc++;
     write_eeprom(0,auxc);
     if(auxc==0){
       auxc=read_eeprom(1);
       auxc++;
       write_eeprom(1,auxc);
       if(auxc==0){
         auxc=read_eeprom(2);
         auxc++;
         write_eeprom(2,auxc);
       }
     }
}
/*******************************************************************************
       Envio de dados para o Transmissor comercial
*******************************************************************************/

void envia_dados(){
int8 auxa,auxb;
//transmite lsb primeiro

    for(auxb=0;auxb<Q_TRANSMITIDO;auxb++){
      rf_out=1;//pulso de start
      delay_us(330);
      rf_out=0;

      for(auxa=0;auxa<64;auxa++){
        restart_wdt();
        if(buffer_Ser[0]&1) {//envia '1'
        rf_out=0;
        delay_us(660);
        rf_out=1;
        delay_us(330);
        rf_out=0;
      }
      else{//envia '0'
      rf_out=0;
      delay_us(330);
      rf_out=1;
      delay_us(660);
      rf_out=0;
      }
      rotate_right(buffer_Ser,8);
     }
     delay_ms(7);//delay entre transmisses
  }//for(auxb=0;auxb<Q_TRANSMITIDO;auxb++)



}//void envia_dados()

/*******************************************************************************
      Rotina de Transmisso do Serial Number para aprendizado do RX
*******************************************************************************/

void transmite_serial_number(){
  int8 sum=0;
  buffer_Ser[0]=SER0;
  sum = sum + SER0;
  buffer_Ser[1]=SER1;
  sum = sum + SER1;
  buffer_Ser[2]=SER2;
  sum = sum + SER2;
  buffer_Ser[3]=SER3;
  sum = sum + SER3;
  buffer_Ser[4]= READ_EEPROM(0); //cnt lower
  sum = sum + buffer_Ser[4];
  buffer_Ser[5]= READ_EEPROM(1); //cnt higher
  sum = sum + buffer_Ser[5];
  buffer_Ser[6]= READ_EEPROM(2); //cnt upper
  sum = sum + buffer_Ser[6];
  buffer_Ser[7] = sum;//somatria

  envia_dados();//envia dados pelo pino de sada do pic (rf_out)

}




/*******************************************************************************
         Rotina de Transmisso de dados
*******************************************************************************/

//rotina de modulaao da transmisso de rf no tx
void transmitir(){//a cada 45 segundos
#define VALOR_TRANSMITIR 19 // VALOR_TRANSMITIR=Tempo desejado em seg /2,304
   SLEEP();//sleep mode
   delay_us(3);
   if(!bt_env_ser){
     while(!bt_env_ser){
       restart_wdt();
       transmite_serial_number();//transmite serial number para aprendizado do rx
       delay_ms(50);
     }
   }
   else{
     if(!TO){
       cntwdt++;
       if(cntwdt>=VALOR_TRANSMITIR){
         cntwdt=0;
         
 //Se uma transmisso no for bem recebida, o rx ficar dessincronizado.
 // Por preveno, envia-se 2 vezes para poder ressincronizar rx, caso seja
 // o caso.
         tx_encripta();//embaralha dados da serial, contador e botes
         envia_dados();//envia dados pelo pino de sada do pic (rf_out)
         incrementa_cnt();//incrementa contador da encriptao
         delay_ms(100);
         tx_encripta();//embaralha dados da serial, contador e botes
         envia_dados();//envia dados pelo pino de sada do pic (rf_out)
         incrementa_cnt();//incrementa contador da encriptao
       }

     }
     else{
      cntwdt=0;//zera no power_on reset
     }
   }
}

void tx_ao_ligar(){
    if(TO){
      cntwdt=0;//zera
      tx_encripta();//embaralha dados da serial, contador e botes
      envia_dados();//envia dados pelo pino de sada do pic (rf_out)
      incrementa_cnt();//incrementa contador da encriptao
      delay_ms(100);
      tx_encripta();//embaralha dados da serial, contador e botes
      envia_dados();//envia dados pelo pino de sada do pic (rf_out)
      incrementa_cnt();//incrementa contador da encriptao
   }
}
/******************************************************************************
                      Rotina Main
*******************************************************************************/
void main() {

   OPTION_REG =  0b00001111;// prescaller para wdt
   WPU = 0XFF; //liga pull ups individual
   CMCON = 7;//desliga comparadores
   TRISIO=0b111110;
   GPIO=0;
   ANSEL=0;
   IOC=0b100000;
   INTCON=0B00001000;//liga interrupo de mudana de estado (pinos dos botes)
   delay_ms(100);
   tx_ao_ligar();//transmite ao ligar a alimentao do tx
/******************************************************************************
                      Lao de Repetio
*******************************************************************************/    
   for(;;){
      transmitir();//rotina de transmisso temporizada (a cada 45 seg)
   }// for(;;)
}//main()

//------------------------------------------------------------------------------
//                           fim do programa

//------------------------------------------------------------------------------

